//+------------------------------------------------------------------+
//|                                                     moexfuts.mq5 |
//|                                    Copyright (c) 2025, Marketeer |
//|                           https://www.mql5.com/ru/articles/16964 |
//+------------------------------------------------------------------+
#property description "List tickers of futures/options from MOEX, filter by security collection and assetcode, create custom symbols"
#property copyright "Copyright (c) 2025, Marketeer"
#property link "https://www.mql5.com/ru/articles/16964"
#property script_show_inputs

#include "moexcore.mqh"
#include "moex2mql5.mqh"
#include "moexref.mqh"

input group "MOEX Assets"
const MOEX_ENGINES Engine = MOEX_ENGINES::no_engines; // implied by/deduced from market
input MOEX_MARKETS Market = futures·forts; // Market (forts by default)
input MOEX_BOARDS Board = forts·RFUD; // Board (for selected Market only!)
input MOEX_SECURITYCOLLECTIONS Collection = MOEX_SECURITYCOLLECTIONS::futures_forts_commodity; // Collection (for selected Market only!)
input string SortByColumn = "openposition"; // SortByColumn (hint:lasttradedate,openposition)
input bool DescendingOrder = true;
input string AssetCodes = "GOLD,SILV,PLT,PLD";
input string DefaultCurrency = "USD";

input group "History"
input datetime From = 0; // From (1970 = very beginning)
input MOEX_INTERVAL Timeframe = M1; // Timeframe (M1 recommended)

input group "Auxiliary"
input bool Logging = false;
input bool Dumping = true;
input bool Test = false;
input uint Timeout = 5; // Timeout (seconds)
input uint Delay = 100; // Delay (between paging requests, ms)
input string Parameters = "iss.meta=off"; // Global Parameters (such as iss.meta=on|off, etc)

// starting requests:
// https://iss.moex.com/iss/engines/futures/markets/forts/boards/RFUD/securities.json?security_collection=futures_forts_commodity&sort_column=lasttradedate
// https://iss.moex.com/iss/engines/futures/markets/forts/boards/RFUD/securities.json?security_collection=futures_forts_commodity&sort_column=openposition&sort_order=desc


template<typename T>
JsValue *Filter(JsValue *data, const int col, T &values[], const bool unique = false)
{
   const int n = ArraySize(values);
   if(!n || !data.length()) return (JsValue *)data;
   JsValue *result = new JsValue(JS_ARRAY);
   string listed[];
   
   for(int i = 0; i < data.length(); i++)
   {
      const T value = data[i][col].get<T>();
      // PrintFormat("%d %s", i, (string)value);
      for(int j = 0; j < n; j++)
      {
         if(value == values[j])
         {
            if(unique)
            {
               ArrayFindOrInsert(listed, value);
            }
            if(!unique || _LastError) // or not found
            {
               result.put(data[i]);
               data.drop(i);
            }
            break;
         }
      }
      
      if(unique && ArraySize(listed) == n) break;
   }
   return result;
}


bool CreateCustomSymbol(const string ticker, MOEX_BOARDS b, MOEX_MARKETS m, MOEX_ENGINES e, const string c, const string name = NULL, const string desc = NULL)
{
   Print("Creating new custom symbol ", ticker, name != NULL ? " (" + name + ")" : "");
   string market, engine;
   Moex::ResolveMoexNames(b, m, e, market, engine);
   
   // Single ticker extensive description from iss/securities/XYZ
   JsValue *about = iss.securities.security(ticker)._get("lang=en"); // use lang=en to detect 'unit' currency
   // if(Dumping) DumpJsonToFile(StringFormat("MOEX/about-%s.json", UseTicker), about);
  
   // Specs for ticker /iss/engines/[engine]/markets/[market]/boards/[board]/securities/XYZ
   JsValue *spec = iss.engines.security(ticker, b, m, e)._get();
   
   if(spec.t == JS_OBJECT) // not an error or null
   {
      if(about.t == JS_OBJECT) // augment specs with description
      {
         spec.merge(about, true);
      }
      
      if(Dumping) DumpJsonToFile(StringFormat("MOEX/symbol-%s.json", ticker), spec);
      // Create custom symbol at engine/market/ticker path
      if(Moex::CreateCustomSymbol(spec, (name == NULL ? ticker : name), market, engine, c, desc))
      {
         Print("Successfully created");
      }
      else
      {
         Print("Failed with error: ", _LastError);
      }
      return true;
   }
   return false;
}

void OnStart()
{
   string codes[];
   StringSplit(AssetCodes, ',', codes);
   
   iss.link.applySettings(MoexLink::Settings(Logging, Dumping, Test, Timeout, Delay, Parameters));
   
   JsValue *list = iss.engines.securities(Market)._security_collection(Collection)._sort_column(SortByColumn)._sort_order((MOEX_SORT_ORDER)DescendingOrder)._get();
   
   const int ac = list["securities"]["columns"].indexOf("ASSETCODE"); // "SECTYPE" - optional prefix in ticker name
   if(ac == -1) return;
   
   JsValue *workset = Filter(list["securities"]["data"], ac, codes, true);
   
   PrintFormat("Found %d tickers of %d", workset.length(), ArraySize(codes));
   
   if(Dumping) DumpJsonToFile("MOEX/futures/filter.json", workset);
   
   if(!ArraySize(codes))
   {
      Alert("Specify AssetCodes to create custom symbols");
      return;
   }

   for(int i = 0; i < workset.length() && !IsStopped(); i++)
   {
      const string ticker = workset[i][0].s; // "SECID"
      
      if(Logging)
      {
         JsValue *range = iss.engines.candleborders(ticker, Board)._get();
         Moex::HistoryRange history[];
         PASS(Moex::GetCandleborders(range, history));
         Print(ticker);
         ArrayPrint(history);
      }

      bool isCustom;
      if(!SymbolExist(ticker + ".iss", isCustom))
      {
         if(!CreateCustomSymbol(ticker, Board, Market, Engine, DefaultCurrency, ticker + ".iss")) continue;
      }
      
      if(!isCustom)
      {
         PrintFormat("%s is not a custom symbol", ticker + ".iss");
         continue;
      }

      // download all candles
      JsValue *data = iss.engines.candles(ticker, Board, Market, Engine)._interval(Timeframe)._from(From)._start(MOEX_ALL)._get();
      MqlRates rates[];
      PASS(Moex::GetRates(data, rates));
      ResetLastError();
      PRTF(CustomRatesReplace(ticker + ".iss", NULL, LONG_MAX, rates));
   }

   delete workset;
}
//+------------------------------------------------------------------+
